<?php 

/**
 Bluetooth Setup Controller
 
 @Copyright 2016 Stefan Rick
 @author Stefan Rick
 Mail: stefan@rick-software.de
 Web: http://www.netzberater.de

 */

/**
 * 
 * @author Stefan Rick
 *
 */
class Bluetooth extends Service {
		
	public $viewname = 'Bluetooth';
	public $bt_speaker_configfile = '/etc/bt_speaker/config.ini';
	protected $pname = 'bluetooth';
	public $btaudio = 'alsa';
	public $pulsesettings = array();
	public $scriptPath = '';
	public $bluetoothsink = '';
	public $bluetoothconnected = false;
	
	public function __construct(){
		parent::__construct();
		$this->scriptPath = dirname(__FILE__).'/../scripts/';
		//Set your Pluginname
		$this->pluginname = _('Bluetooth');
		$this->registerLocale(dirname(__FILE__).'/../locale', 'bluetooth');
		$this->getVersion();
		if($this->info->version < '2.28'){
			$this->view->message[] = str_replace(array('NEEDEDVERSION', 'YOURVERSION'),array('2.28', $this->info->version),_t('You need at least Max2Play Version NEEDEDVERSION. Your Max2Play Version is YOURVERSION. Please update (on settings page) before using this addon.'));
		}
		
		if($this->checkLicense(true) == false)
			return true;
		
		if($_REQUEST['ajax'] == 1 && $_REQUEST['action'] == 'install'){
			//Function to get Progress of Installation
			$this->_install(1);
			ob_end_clean();
			echo implode('<br />', $this->view->message);
			ob_flush();
			die();
		}
		
		$this->_checkinstall();
		
		if(isset($_REQUEST['action'])){
			if($_REQUEST['action'] == 'install'){
				$this->_install();
			}
			
			if($_REQUEST['action'] == 'uninstall'){
				$this->_uninstall();
			}
			
			if($this->btaudio == 'bluealsa'){
			    // Only for Stretch Image without Pulse
			    // Bluealsa (Output) & BT-Speaker (Input)
			    if($_REQUEST['action'] == 'save_btspeaker'){
			        $this->_get_btspeaker();
			        $this->_save_btspeaker();
			    }
			    if ($_REQUEST['action'] == 'start_btspeaker') {
			        // Backgroundfile is set as output in init.d Script!
			        $this->view->message[] = $this->start('bt_speaker', $command = 'systemctl start bluealsa_aplay', $statusname = 'bluealsa-aplay', $rootstart = true);
			    }
			    
			    if ($_REQUEST['action'] == 'stop_btspeaker') {
			        $this->view->message[] = $this->stop('bt_speaker', $command = 'systemctl stop bluealsa_aplay', $statusname = 'bluealsa-aplay', $rootstop = true);
			    }			    
			}
			if($_REQUEST['action'] == 'installbluetoothalsasbs'){
			    $this->_installbluetoothalsasbs();
			}
			
			if($_REQUEST['action'] == 'installbluetoothsbs'){
				$this->_installbluetoothsbs();
			}			
			if($_REQUEST['action'] == 'resetbluetoothdevice'){
			    $this->_resetBTDevice();
			}
			
			if($_REQUEST['action'] == 'scan'){				
				$this->_scan();				
			}
			if($_REQUEST['action'] == 'squeezelitebluetooth'){				
				$this->_installSqueezeliteBluetooth();				
			}
			if(strpos($_REQUEST['action'], 'connect') === 0){
				$this->_connect(explode('_', $_REQUEST['action']));
			}
			if(strpos($_REQUEST['action'], 'disconnect') === 0){
				$this->_disconnect(explode('_', $_REQUEST['action']));
			}
			if(strpos($_REQUEST['action'], 'remove') === 0){
				$this->_remove(explode('_', $_REQUEST['action']));
			}
			if($_REQUEST['action'] == 'save'){				
				$this->selectAutostart(isset($_REQUEST['PLUSEBLUETOOTH_AUTOSTART_SBS']) ? 1 : 0, true, 'PLUSEBLUETOOTH_AUTOSTART_SBS');
				$this->selectAutostart(isset($_REQUEST['autostart']) || isset($_REQUEST['PLUSEBLUETOOTH_AUTOSTART_SBS']) ? 1 : 0);
				$this->_savePulseSettings();
				$this->_checkinstall();
			}
			if($_REQUEST['action'] == 'savealsasbs'){
			    $this->selectAutostart(isset($_REQUEST['ALSABLUETOOTH_AUTOSTART_SBS']) ? 1 : 0, true, 'ALSABLUETOOTH_AUTOSTART_SBS');
			    $this->selectAutostart(isset($_REQUEST['autostart']) || isset($_REQUEST['ALSABLUETOOTH_AUTOSTART_SBS']) ? 1 : 0);
			    // kill existing alsabluetoothsbs process if autostart = 0
			    if(!isset($_REQUEST['ALSABLUETOOTH_AUTOSTART_SBS'])){
			        $this->writeDynamicScript(array('killall alsabluetoothsbs'));
			    }
			    $this->_saveAlsaSBSSettings();
			    $this->_checkinstall();
			}
		}		
		if($this->view->installed){
			//Get Debug Info
			$this->_getDeviceList();
			$this->_getAllLogs();
			$this->view->autostart = $this->checkAutostart($this->pname, true);
			$this->view->PLUSEBLUETOOTH_AUTOSTART_SBS= $this->checkAutostart('PLUSEBLUETOOTH_AUTOSTART_SBS', true);
			$this->view->ALSABLUETOOTH_AUTOSTART_SBS= $this->checkAutostart('ALSABLUETOOTH_AUTOSTART_SBS', true);
			
			if($this->btaudio == 'bluealsa'){
			    $this->_get_btspeaker();
			}
		}
		
		$this->showHelpSidebar();
	}
	
	/**
	 * Save BT-Speaker Settings
	 * Autostart, Soundcard, etc.
	 * /etc/bt_speaker/config.ini
	 * 
	 */
	private function _save_btspeaker(){
	    $this->getSoundDevices();
	    // use systemctl to Start/Stop/Autostart 
	    $this->selectAutostart(isset($_REQUEST['autostart_bt_speaker']) ? 1 : 0, 'systemd', 'bluealsa_aplay');	    
	    
	    if($_REQUEST['btspeaker_device']){
	        if(isset($this->view->soundDevices[$_REQUEST['btspeaker_device']])){
	            $device = $this->view->soundDevices[$_REQUEST['btspeaker_device']];
	            
	            $this->saveConfigFileParameter('/opt/max2play/audioplayer.conf', 'BLUEALSA_PLAY_MIXER', $device['mixer_control']);
	            $this->saveConfigFileParameter('/opt/max2play/audioplayer.conf', 'BLUEALSA_PLAY_AUDIOCARD', 'hw:'.$device['card'].','.$device['device']);
	            
	            if($this->view->bt_speaker_status){
	                // Service Running -> restart 
	                $this->view->message[] = $this->view->bt_speaker_viewname.' Service restarted';
	                $this->writeDynamicScript('systemctl restart bluealsa_aplay');
	            }
	        }
	    }
	    
	}
	
	private function _get_btspeaker(){
	    // Check for existing service Script
	    if(!file_exists('/etc/systemd/system/bluealsa_aplay.service')){
	        $this->writeDynamicScript(array("cp $this->scriptPath/bluealsa_aplay.service /etc/systemd/system/"));
	    }
	    $this->view->autostart_bt_speaker = $this->checkAutostart('bluealsa_aplay', 'systemd');
	    $this->view->bt_speaker_status = $this->status('bluealsa-aplay');
	    $this->view->bt_speaker_viewname = 'Bluetooth-Speaker';
	    
	    $this->getSoundDevices();
	    $audiocard = $this->getConfigFileParameter('/opt/max2play/audioplayer.conf', 'BLUEALSA_PLAY_AUDIOCARD');
	    if(!$audiocard){
	        $this->saveConfigFileParameter('/opt/max2play/audioplayer.conf', 'BLUEALSA_PLAY_AUDIOCARD', 'hw:0,0');
	    }
	    $this->view->btspeakerconfig_position = str_replace('hw:', '', $audiocard);
	}
	
	/**
	 * get Precompiled Squeezlite from Max2Play Servers 
	 */
	private function _installSqueezeliteBluetooth(){
		$this->view->message[] = nl2br($this->writeDynamicScript(array(
					'wget cdn.max2play.com/squeezeliteBT.zip -O /opt/max2play/cache/squeezeliteBT.zip',
					'rm /opt/squeezelite/*',
					'unzip -u /opt/max2play/cache/squeezeliteBT.zip -d /opt/squeezelite/')));
		$this->view->message[] = _t('Squeezelite Updated - Now add "-B" to the command line options of Squeezelite to attach its power status to the Bluetooth-Speakers Connection status.');
		return true;
	}
	
	/**
	 * save special settings for Pulseaudio
	 */
	private function _savePulseSettings(){
		
		if($this->pulsesettings['module-rescue-streams'] == 0 && isset($_REQUEST['module-rescue-streams'])){
			// Enable Recover Stream -> Enable null sink and set default
			$this->writeDynamicScript(array('sed -i "s/^#load-module module-rescue-streams$/load-module module-rescue-streams/;s/^load-module module-null-sink$/#load-module module-null-sink/" /etc/pulse/default.pa', 
					'sudo su -l '.$this->getSystemUser().' -c "pactl load-module module-rescue-streams"',
					'sudo su -l '.$this->getSystemUser().' -c "pactl unload-module module-null-sink"'));			
			$this->view->message[] = _t('Added module-rescue-streams to Pulseaudio');
		}elseif($this->pulsesettings['module-rescue-streams'] == 1 && !isset($_REQUEST['module-rescue-streams'])){
			// Do Not Recover Streams
			$this->writeDynamicScript(array('sed -i "s/^load-module module-rescue-streams$/#load-module module-rescue-streams/;s/^#load-module module-null-sink$/load-module module-null-sink/" /etc/pulse/default.pa', 
					'sudo su -l '.$this->getSystemUser().' -c "pactl unload-module module-rescue-streams"',
					'sudo su -l '.$this->getSystemUser().' -c "pactl load-module module-null-sink"',
					'sudo su -l '.$this->getSystemUser().' -c "pactl set-default-sink null"'));
			$this->view->message[] = _t('Removed module-rescue-streams from Pulseaudio');
		}
		
		if(isset($_REQUEST['BLUETOOTH_MACADDRESS_MAINPLAYER']) && $this->view->BLUETOOTH_MACADDRESS_MAINPLAYER != $_REQUEST['BLUETOOTH_MACADDRESS_MAINPLAYER']){
			$this->saveConfigFileParameter('/opt/max2play/audioplayer.conf', 'BLUETOOTH_MACADDRESS_MAINPLAYER', $_REQUEST['BLUETOOTH_MACADDRESS_MAINPLAYER']);
		}
		if($this->view->PLUSEBLUETOOTH_DEBUG != $_REQUEST['PLUSEBLUETOOTH_DEBUG']){
			$this->saveConfigFileParameter('/opt/max2play/autostart.conf', 'PLUSEBLUETOOTH_DEBUG', $_REQUEST['PLUSEBLUETOOTH_DEBUG']);
		}
		
		return true;
	}
	
	/**
	* save special settings for Alsa with Squeezebox Server
	*/
	private function _saveAlsaSBSSettings(){
	    if(isset($_REQUEST['BLUETOOTH_MACADDRESS_MAINPLAYER']) && $this->view->BLUETOOTH_MACADDRESS_MAINPLAYER != $_REQUEST['BLUETOOTH_MACADDRESS_MAINPLAYER']){
	        $this->saveConfigFileParameter('/opt/max2play/audioplayer.conf', 'BLUETOOTH_MACADDRESS_MAINPLAYER', $_REQUEST['BLUETOOTH_MACADDRESS_MAINPLAYER']);
	    }
	    if($this->view->ALSABLUETOOTH_DEBUG != $_REQUEST['ALSABLUETOOTH_DEBUG']){
	        $this->saveConfigFileParameter('/opt/max2play/autostart.conf', 'ALSABLUETOOTH_DEBUG', $_REQUEST['ALSABLUETOOTH_DEBUG']);
	    }
	}
	
	private function _scan(){
		$script = array();		
		if($this->btaudio == 'pulse'){
			$this->view->message[] = _t('Using BluetoothCTL Manager');
			$script[] = "/usr/bin/bt-adapter -d &";
			$script[] = "sleep 20";
			$script[] = "killall bt-adapter";
			$shellanswer = $this->writeDynamicScript($script);			
		}else{
		    $script[] = "export XDG_RUNTIME_DIR=/run/user/1000";
		    $script[] = 'echo -e \'discoverable on \nquit\' | /usr/bin/bluetoothctl';
		    $script[] = "/usr/bin/hcitool scan";			
			$shellanswer = $this->writeDynamicScript($script);
			# Result looks like this:
			# Scanning ...
			#    40:EF:4C:FF:03:22	JBL GO
			if(preg_match_all('=([0-9ABCDEF\:]{17})[ \t]+(.*)$=im', $shellanswer, $matches)){
				$count = count($matches[1]);
				for($i = 0; $i < $count; $i++){
					$this->view->scandevices[$matches[1][$i]] = array('address' => $matches[1][$i], 'name' => $matches[2][$i]);
				}
			}
		}
				
		$this->view->message[] = _t('Scanning Complete');
		return true;
	}
	
	private function _checkinstall(){
		$this->getLinuxVersion();
		if(isset($this->info->linux[1]) && $this->info->linux[0] == "Raspbian" && ($this->info->linux[1] == "jessie")){
			$this->btaudio = 'pulse';
			//Pulsesetting load-module module-rescue-streams
			$this->pulsesettings['module-rescue-streams'] = trim(shell_exec('grep -e "^load-module module-rescue-streams" /etc/pulse/default.pa | wc -l'));
			
			// Disable Bluez5 to prevent Connection Errors
// 			$unloadmodulebluez5discover = trim(shell_exec('grep -e "^unload-module module-bluez5-discover" /etc/pulse/default.pa | wc -l'));
// 			if($unloadmodulebluez5discover == 0){
// 				$this->writeDynamicScript(array('sed -i "s/load-module module-bluetooth-discover/load-module module-bluetooth-discover\nunload-module module-bluez5-discover/" /etc/pulse/default.pa'));
// 				$this->view->message[] = _t('Disabled Bluez5 Module Discover to prevent Connection Errors');
// 			}
			
			if(trim(shell_exec('dpkg -s bluez-tools | grep "install ok" | wc -l')) > 0){
			    $this->view->installed = 1;
			}
		}elseif($this->info->linux[1] == "stretch" || $this->info->linux[1] == "buster"){		    
		    $this->btaudio = 'bluealsa';
		    if(file_exists('/usr/bin/bluealsa')){
		        $this->view->installed = 1;
		        // Also check Service File for downwards compatibility
		        if(trim($this->shell_exec('grep "Restart=on-failure" /etc/systemd/system/bluealsa.service | wc -l')) == 0){
		        	$this->writeDynamicScript(array('cp ' . $this->scriptPath. 'bluealsa.service /etc/systemd/system/;systemctl daemon-reload;systemctl enable bluealsa;systemctl start bluealsa;'));
		        	$this->view->message[] = _t('Fixed Service File for Bluealsa compatibility for previous installs.');
		        }
		    }
		}else{
			$this->btaudio = 'alsa';
			if(trim(shell_exec('dpkg -s bluez-tools | grep "install ok" | wc -l')) > 0){
			    $this->view->installed = 1;
			}
		}
		
		
		if($this->view->installed == 1){
		    $this->view->BLUETOOTH_MACADDRESS_MAINPLAYER = $this->getConfigFileParameter('/opt/max2play/audioplayer.conf', 'BLUETOOTH_MACADDRESS_MAINPLAYER');
		    
		    $this->view->PLUSEBLUETOOTH_DEBUG = $this->getConfigFileParameter('/opt/max2play/autostart.conf', 'PLUSEBLUETOOTH_DEBUG');
		    $this->view->ALSABLUETOOTH_DEBUG = $this->getConfigFileParameter('/opt/max2play/autostart.conf', 'ALSABLUETOOTH_DEBUG');
		    
		    // Get current MAC ADDRESS
		    $hwaddr = explode('\n', shell_exec('LANG=LC && /sbin/ip addr | /usr/bin/tr "\n" " " | grep -o "ether [a-f0-9:]\+ brd ff:ff:ff:ff:ff:ff \+inet [0-9.]\+" | grep -o "[a-f0-9:]\{17\}" | grep -v "ff:ff:ff:ff:ff:ff"'));
		    $this->view->hwaddress = $hwaddr[0];
		}
		
		// Get current Bluetoothsink
		$this->bluetoothsink = $this->getConfigFileParameter('/opt/max2play/options.conf', 'BLUETOOTHSINK');
		return true;
	}
	
	private function _disconnect($device){		
	    if($this->btaudio == 'pulse' || $this->btaudio == 'bluealsa'){
			$this->view->message[] = _t('Using BluetoothCTL Manager');
			$script[] = "export XDG_RUNTIME_DIR=/run/user/1000";
			$script[] = "/usr/bin/bluetoothctl << EOF\ndisconnect ".$device[1]."\nEOF\n";		  
		}else{
			$script[] = "/usr/bin/bt-audio -d ".$device[1];
		}		
		$this->view->message[] = nl2br(trim(preg_replace('=(.*\])|(.*\[K)|(.*#)|(spawn .*)|(\^M)=i','', $this->writeDynamicScript($script))));
		sleep(2);
		return true;
	}
	
	private function _remove($device){
		$script[] = "/usr/bin/bt-device -r ".$device[1]." 2>&1";
		$this->view->message[] = nl2br($this->writeDynamicScript($script));
		sleep(2);
		return true;
	}

	/**
	 * 
	 * @param array $device 
	 * 	[1] => address 
	 * 	[2] => name
	 * @return boolean
	 */
	private function _connect($device){		
		/*Switch to EXPECT to get all Errors and better Script handling! Make sure it is installed! */
	    if($this->btaudio == 'bluealsa' && file_exists('/usr/bin/expect')){
            // New Handling for Debian Stretch!
	        $script[] = "export XDG_RUNTIME_DIR=/run/user/1000";
	        $script[] = $this->scriptPath."connectbluealsa.exp ".$device[1]." pi";
	        $script[] = "/usr/bin/bt-device -i '".urldecode($device[2])."'";
	        $script[] = 'sed -i "s/ device [0-9A-F:]\{17\}/ device '.$device[1].'/" /etc/asound.conf';
	        $this->view->message[] = _t('<b>If you connected Bluetooth Speakers, make sure to set the soundcard of your audioplayers to bluetooth as audio output! You need to restart your Audioplayer if the Bluetooth Device changed.</b>');
	        $this->view->message[] = _t('<b>If you connected a Smartphone to stream to Squeezebox Server, NO further setting to your soundcard is needed (do NOT set your soundcard to bluealsa)!</b>');
	        $this->view->message[] = nl2br(trim(preg_replace("=\n[ \r\t\n]+=", "\n", preg_replace('=(.*\])|(.*\[K)|(.*#)|(spawn .*)|(\^M)|([0-9a-f\-]{36})=i','', $this->writeDynamicScript($script)))));
	        
	    }elseif($this->btaudio == 'pulse' && file_exists('/usr/bin/expect')){
			$script[] = "export XDG_RUNTIME_DIR=/run/user/1000";
			$script[] = $this->scriptPath."connect.exp ".$device[1]." pi";
			$script[] = "/usr/bin/bt-device -i '".urldecode($device[2])."'";
			$this->view->message[] = _t('<b>If you connected Bluetooth Speakers, make sure to set the soundcard of your audioplayers to PULSE (advanced settings Squeezelite) to get them to play with Bluetooth!</b>');
			$this->view->message[] = _t('<b>If you connected a Smartphone to stream to Squeezebox Server, NO further setting to your soundcard is needed (do NOT set your soundcard to pulse)!</b>');
			$this->view->message[] = nl2br(trim(preg_replace("=\n[ \r\t\n]+=", "\n", preg_replace('=(.*\])|(.*\[K)|(.*#)|(spawn .*)|(\^M)|([0-9a-f\-]{36})=i','', $this->writeDynamicScript($script)))));
			sleep(5);
			$script = array();
			//TODO: always get correct card...
			$script[] = 'su -l pi -c "pactl set-card-profile 1 a2dp"';
			$script[] = 'su -l pi -c "pactl set-default-sink bluez_sink.'.str_replace(':','_',$device[1]).'"';
			$this->view->message[] = nl2br($this->writeDynamicScript($script));
			$this->saveConfigFileParameter('/opt/max2play/options.conf', 'BLUETOOTHSINK', 'bluez_sink.'.str_replace(':','_',$device[1]));
			
		}elseif($this->btaudio == 'pulse'){
			$this->view->message[] = _t('Using BluetoothCTL Manager');
			$script[] = "export XDG_RUNTIME_DIR=/run/user/1000";
			$script[] = "/usr/bin/bluetoothctl << EOF\nscan on\nEOF\n";
			$script[] = "sleep 3";
			$script[] = "/usr/bin/bluetoothctl << EOF\ntrust ".$device[1]."\nEOF\n";
			$script[] = "/usr/bin/bluetoothctl << EOF\npair ".$device[1]."\nEOF\n";
			$script[] = "/usr/bin/bluetoothctl << EOF\nconnect ".$device[1]."\nEOF\n";
			$script[] = "/usr/bin/bt-device -i '".urldecode($device[2])."'";
			$this->view->message[] = _t('<b>If you connected Speakers, make sure to set the soundcard of your audioplayers to PULSE (advanced settings Squeezelite) to get them to play with Bluetooth!</b>');
			$this->view->message[] = nl2br(preg_replace('=([^ \n]*(094|092|091)[^ \n]+)=i','', preg_replace('/[^a-zA-Z0-9\n[:space:]:]/i','', $this->writeDynamicScript($script))));
			sleep(5);
			$script = array();
			//TODO: always get correct card...
			$script[] = 'su -l pi -c "pactl set-card-profile 1 a2dp"';
			$script[] = 'su -l pi -c "pactl set-default-sink bluez_sink.'.str_replace(':','_',$device[1]).'"';
			$this->view->message[] = nl2br($this->writeDynamicScript($script));
			$this->saveConfigFileParameter('/opt/max2play/options.conf', 'BLUETOOTHSINK', 'bluez_sink.'.str_replace(':','_',$device[1]));
		}else{
			//Connect to Device
			$script[] = "/usr/bin/bt-device -c ".$device[1];
			//Trust Device
			$script[] = "/usr/bin/bt-device --set '".urldecode($device[2])."' Trusted 1";
			//Get Information
			$script[] = "/usr/bin/bt-device -i '".urldecode($device[2])."'";
			//Connect Audio
			$script[] = "/usr/bin/bt-audio -c ".$device[1];
			//Replace DeviceID in asound Config
			$script[] = 'sed -i "s/^[ ]*device [0-9A-F:]\{17\}[ ]*$/   device '.$device[1].'/" /etc/asound.conf';
			$this->view->message[] = nl2br($this->writeDynamicScript($script));
			$this->view->message[] = _t('<b>YOU PROBABLY NEED TO RESTART your AUDIOPLAYER (e.g. Squeezelite) to get them to work with Bluetooth!</b>');
		}
		
		
		return true;
	}
	
	private function _getDeviceList(){
		$this->view->connecteddevices = false;
		$script[] = "/usr/bin/bt-device -l";
		$shellanswer = $this->writeDynamicScript($script);
		$useExpect = false;
		if($this->btaudio == 'bluealsa' && file_exists('/usr/bin/expect')){
		    // This offers better reliability - no GLib-CRITICAL Error
		    $useExpect = true;
		}
		if(preg_match_all('=(.*) \(([0-9ABCDEF\:]{17})\)=', $shellanswer, $matches)){
			$count = count($matches[1]);
			for($i = 0; $i < $count; $i++){			
				$this->view->connecteddevices[$matches[1][$i]] = array('address' => $matches[2][$i], 'name' => $matches[1][$i]);
				if($useExpect){
				    $shellanswer = $this->writeDynamicScript(array($this->scriptPath."connectstatus.exp ".$matches[2][$i]));
				}else{
				    $shellanswer = $this->writeDynamicScript(array("/usr/bin/bt-device -i '".$matches[1][$i]."'"));
				}
				$this->view->connecteddevices[$matches[1][$i]]['info'] = $shellanswer;
				$this->view->connecteddevices[$matches[1][$i]]['connected'] = (strpos($shellanswer, 'Connected: 1') !== FALSE || strpos($shellanswer, 'Connected: yes') !== FALSE) ? 1 : 0;

    			if($this->view->connecteddevices[$matches[1][$i]]['connected'] == 1){
    			    $this->bluetoothconnected = true;
    			}
			}			
		}
		return true;		
	}
	
	private function _getParameters(){
		// is Installed? Check Package and Script Files
		// run hciconfig to get info on Dongle
		return true;
	}
	
	private function _install($ajax = 0){		
		if($ajax == 0){
			ignore_user_abort(true);
			set_time_limit(3600);
			$this->view->message[] = _t('Bluetooth install started');	
			if($this->getProgressWithAjax('/opt/max2play/cache/install_bluetooth.txt', 1, 1)){
				$shellanswer = $this->writeDynamicScript(array($this->scriptPath."install.sh >> /opt/max2play/cache/install_bluetooth.txt &"), false, true);
			}
		}else{
			//Get only last 20 Lines
			$status = $this->getProgressWithAjax('/opt/max2play/cache/install_bluetooth.txt',0, 0, 20);
			$this->view->message[] = nl2br($status);
			if(strpos($status, 'Finished') !== FALSE){
				shell_exec('rm /opt/max2play/cache/install_bluetooth.txt');
				$this->selectAutostart(1);
			}
		}
		return true;
	}
	
	/**
	 * Installer for Bluetooth Support for ALSA on Squeezebox Server
	 * @param number $ajax
	 * @return boolean
	 */
	private function _installbluetoothalsasbs($ajax = 0){
	    ignore_user_abort(true);
	    set_time_limit(3600);
	    $this->view->message[] = _t('Install Bluetooth support for Squeezebox Server');
	    $this->view->message[] = nl2br($this->writeDynamicScript(array($this->scriptPath."sbs_bluetooth.sh")));
	    $this->selectAutostart(0);	    
	    $this->_saveAlsaSBSSettings();
	    // Setup Autostart for everything...
	    $this->selectAutostart(1, true, 'ALSABLUETOOTH_AUTOSTART_SBS');
	    $this->selectAutostart(1);
	    $this->_checkinstall();
	    return true;
	}
	
	/**
	 * Installer for Bluetooth Support on Squeezebox Server
	 * @param number $ajax
	 * @return boolean
	 */
	private function _installbluetoothsbs($ajax = 0){		
		ignore_user_abort(true);
		set_time_limit(3600);
		$this->view->message[] = _t('Install Bluetooth support for Squeezebox Server');		
		$this->view->message[] = nl2br($this->writeDynamicScript(array($this->scriptPath."sbs_bluetooth.sh")));
		$this->selectAutostart(0);
		unset($_REQUEST['module-rescue-streams']);
		$this->_savePulseSettings();
		// Setup Autostart for everything...
		$this->selectAutostart(1, true, 'PLUSEBLUETOOTH_AUTOSTART_SBS');
		$this->selectAutostart(1);
		$this->_checkinstall();
		return true;
	}
	
	private function _uninstall(){
		$this->view->message[] = _t('Bluetooth uninstall started');
		$script = array();
		// optional: apt-get remove bluetooth -y
		if($this->btaudio == 'bluealsa'){
		    if(file_exists('/usr/bin/bluealsa')){
		      $script[] = 'rm -R /usr/bin/bluealsa;systemctl stop bluealsa;rm /etc/systemd/system/bluealsa.service;systemctl daemon-reload;';
		    }
		}else{
		    $script[] = 'echo "Y" | apt-get purge bluez-utils bluez-tools bluez -y';
		    $script[] = 'rm -R /var/lib/bluetooth';
		}
		$this->view->message[] = $this->formatMessageOutput($this->writeDynamicScript($script));
		$this->_checkinstall();
		$this->_getParameters();
		return true;
	}
	
	/**
	 * Try to Reset BT-Device
	 * gives Error on RPI3 if Build-In Bluetooth hangs
	 */
	private function _resetBTDevice(){
	    $this->view->message[] = _t('Reset Bluetooth Device hci0');
	    $result = $this->writeDynamicScript(array('hciconfig hci0 reset'));
	    $this->view->message[] = $this->formatMessageOutput($result);
	    return true;
	}
	
	/**
	 * get some Debug Output and save it for view
	 */
	private function _getAllLogs(){		
		$out = array();
		$this->view->hintalert = false;
		$this->view->hints = array();
		
		$out['Bluetooth Installation'] = '';
		$out['Bluetooth Dongle'] = $this->writeDynamicScript(array('hciconfig'));
		if($out['Bluetooth Dongle']){
			$this->view->hints['Bluetooth-Dongle'] = _t('available');
			// TODO: check for "UP RUNNING PSCAN" Problem with RPI3 build-in bluetooth: hciconfig hci0 reset -> timeout AND hcitool dev -> no device after some time
			if(strpos($out['Bluetooth Dongle'],'UP RUNNING') === FALSE){
			    $this->view->hints['Bluetooth-Dongle'] = _t('Not working correctly - Restart Device!');
			    $this->view->hintalert = true;
			}
		}else{
			$this->view->hints['Bluetooth-Dongle'] = _t('NOT available');
			$this->view->hintalert = true;
		}
		
		if($this->view->connecteddevices)
			foreach($this->view->connecteddevices as $device){
				$out['DEVICE '.$device['name']] = $device['info'];
			}		
		if($this->btaudio == 'pulse'){
			$out['PACMD LIST | grep active'] = $this->writeDynamicScript(array('su -l pi -c "pacmd list | grep active"'));
			$out['PACMD LIST | grep sink'] = $this->writeDynamicScript(array('su -l pi -c "pacmd list | grep sink"'));
			
			if($this->bluetoothsink != ''){
			     $this->view->hints['Bluetoothsink Available'] = $this->writeDynamicScript(array('su -l pi -c "pacmd list-sink-inputs | grep '.$this->bluetoothsink.' | wc -l"'));
			     if($this->view->hints['Bluetoothsink Available'] == 0 && $this->bluetoothconnected == true){
			         $this->view->hintalert = true;
			         $this->view->hints['Bluetoothsink Available'] = _t('Restart Device! Pulseaudio might have crashed');
			     }elseif($this->view->hints['Bluetoothsink Available'] != 0){
			         $this->view->hints['Bluetoothsink Available'] =  _t('Yes');
			     }else{
			         $this->view->hints['Bluetoothsink Available'] =  _t('No');
			     }
			         
			}else{
			    $this->view->hints['Bluetoothsink Available'] = _t('Not Setup');
			}
			
			$this->view->xserverfound=trim($this->writeDynamicScript(array('ps -Al | grep lxsession | wc -l')));
			if($this->view->xserverfound === "0"){
				$this->view->message[] = _t('No running X-Server found! Bluetooth needs Pulseaudio and a active X-Session to run! Please activate Autostart...');				
			}
			$out['X-SERVER RUNNING'] = $this->view->xserverfound;
			$out['PULSEAUDIO RUNNING'] = shell_exec('ps -Al | grep pulse');
			if(file_exists('/tmp/pulsebluetooth'))
				$out['BLUETOOTH SQUEEZEBOX SERVER AUTOSTART'] = shell_exec('tail /tmp/pulsebluetooth');
			
			if($out['PULSEAUDIO RUNNING'])
				$this->view->hints['Pulseaudio'] = true;
			else{
				$this->view->hints['Pulseaudio'] = false;
				$this->view->hintalert = true;
			}
			
			if(isset($out['BLUETOOTH SQUEEZEBOX SERVER AUTOSTART']) && strlen($out['BLUETOOTH SQUEEZEBOX SERVER AUTOSTART']) > 1)
				$this->view->hints['Squeezebox Server Autoplay'] = true;
			
			// Profile Switcher for existing Connections
			$out['PACMD LIST-CARDS | DEBUG COMPLETE'] = $this->writeDynamicScript(array('su -l pi -c "pacmd list-cards"'));
		}
		if($this->btaudio == 'bluealsa'){
		    // Check for running Bluealsa
		    $this->view->bluealsarunning = trim($this->writeDynamicScript(array('ps -Al | grep bluealsa | wc -l')));
		    if($this->view->bluealsarunning === "0"){
		        $this->view->hintalert = true;
		        $out['BLUEALSA ERROR'] = $this->writeDynamicScript(array('systemctl status bluealsa'));
		    }
		    $this->view->hints['Bluealsa Running'] = $this->view->bluealsarunning;
		    $out['BLUEALSA RUNNING'] = $this->view->bluealsarunning;
		    if(file_exists('/tmp/alsabluetoothsbs'))
		        $out['BLUETOOTH SQUEEZEBOX SERVER AUTOSTART'] = shell_exec('tail /tmp/alsabluetoothsbs');
		}
		
		//Check for additional Soundcard and settings to use Pulse in Squeezelite / Shairport
		// aplay -l
		
		$this->view->debug = $out;
	}
	
	/**
	 * get Playable Sounddevices for BT_SPEAKER
	 */
	public function getSoundDevices(){	    
	    $this->view->soundDevices = array();
	    $aplayOutput = $this->writeDynamicScript(array('aplay -l | grep "card"'));
	    //card 0: ALSA [bcm2835 ALSA], device 1: bcm2835 ALSA [bcm2835 IEC958/HDMI]
	    //card 1: IQaudIODAC [IQaudIODAC], device 0: IQaudIO DAC HiFi pcm512x-hifi-0 []
	    preg_match_all('=^card ([0-9]): ([^ ]*) \[.*?\], device ([0-9]): (.*)=im', $aplayOutput, $aplayMatches);
	    $count = count($aplayMatches[1]);
	    $RPI_Functions = $this->Rpi_functions();
	    for($i = 0; $i < $count; $i++){
	        if($aplayMatches[2][$i] != ''){
	            $card = $aplayMatches[1][$i];
	            $device = $aplayMatches[3][$i];
	            $this->view->soundDevices[$card.','.$device] = array(
    	            'name' => $aplayMatches[2][$i],
    	            'card' => $aplayMatches[1][$i],
    	            'device' => $aplayMatches[3][$i],
    	            'description' => $aplayMatches[4][$i],
    	        );
	        
    	        // get amixercard by Card
	            foreach($RPI_Functions->usbSoundCards as $amcard){
	                if($amcard['amixercard'] == $aplayMatches[2][$i] && isset($amcard['mpd_mixer_control']) && $amcard['mpd_mixer_control'] != ''){
	                    $this->view->soundDevices[$card.','.$device]['mixer_control'] = $amcard['mpd_mixer_control'];
    	                break;
    	            }
    	        }
    	        if (!isset($this->view->soundDevices[$card.','.$device]['mixer_control']))
    	            $this->view->soundDevices[$card.','.$device]['mixer_control'] = 'PCM';
	        }
	    }
	    
	    return true;
	}
	
	public function showHelpSidebar(){
		global $helpSidebar;
		$helpSidebar['title'] = _t('Help - Bluetooth');
		$helpSidebar['content'] = _t('<ul><li>Install Bluetooth and connect to your Bluetooth Speakers.</li><li>Important: after connecting to your Speakers you must choose Pulse / Bluetoothspeaker as sound output for your audioplayers like Squeezelite or Shairport!</li></ul>');
		if($this->view->hints){ 
			$helpSidebar['content'] .= '<div class="ui-widget"><div class="ui-state-'.(($this->view->hintalert) ? 'error' : 'highlight').' ui-corner-all" style="margin-bottom: 10px; padding: 0.4em .7em;border:none;'.((!$this->view->hintalert) ? 'background:#98ff90;' : ''). '"><p style="'.(($this->view->hintalert) ? 'color:#fff;' : ''). '"><span class="ui-icon ui-icon-info" style="float: left; margin-right: .3em;"></span>';
			$helpSidebar['content'] .= _t('Settings & Hardware');
			foreach($this->view->hints as $name => $hint) $helpSidebar['content'] .= "<br />".$name.': '.$hint;			
			$helpSidebar['content'] .= ' </p></div></div>';
		}				
		$helpSidebar['wikilink'] = 'https://www.max2play.com/en/wiki/using-the-plugin-bluetooth/';
		return true;
	}
}

$bt = new Bluetooth();

//This Line includes the View-Script -> it should have the same name as your class
include_once(dirname(__FILE__).'/../view/setup.php');

