/**
 * alsabluetoothsbs.c
 * Stefan Rick Max2Play.com
 * Compiled with gcc -o alsabluetoothsbs alsabluetoothsbs.c
 * 30-05-2018
 */

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/select.h>
#include <sys/socket.h>
#include <arpa/inet.h> 
#include <sys/inotify.h>
#include <limits.h>

#define MAX_EVENTS 1024 /*Max. number of events to process at one go*/
#define LEN_NAME 16 /*Assuming that the length of the filename won't exceed 16 bytes*/
#define EVENT_SIZE  ( sizeof (struct inotify_event) ) /*size of one event*/
#define BUF_LEN     ( MAX_EVENTS * ( EVENT_SIZE + LEN_NAME )) /*buffer to store the data of events*/
#define _(x) x
#define SIZE 100

int ret;
int btstatus = -1;
int debug = 0;
int poweroffdelay = 1;

char macparameter[] = "BLUETOOTH_MACADDRESS_MAINPLAYER";
char macaddress[30];
char *lastmode, *answer;
char lastpath[256];
char server_reply[500];

void getPlayerParameters() {
	char *token;
	char *search = "=";

	static const char filename[] = "/opt/max2play/audioplayer.conf";
	FILE *file = fopen(filename, "r");
	if (file != NULL) {
		char line[528];
		while (fgets(line, sizeof line, file) != NULL) {
			// Parameter Name
			token = strtok(line, search);
			if (strcmp(token, macparameter) == 0) {
				// Parameter Value
				token = strtok(NULL, search);
				token[strcspn(token, "\n")] = 0;
				strcpy(macaddress, token);
			}
		}
		fclose(file);
	}
	return;
}

void show_error(char *s) {
	fprintf(stderr, "%s\n", s);
}

/**
 Send to Socket of Squeezebox Server
 */
char * sendToSqueezeboxServer(char *message) {
	int sock, n;
	struct sockaddr_in server;

	//Create socket
	sock = socket(AF_INET, SOCK_STREAM, 0);
	if (sock == -1) {
		printf("Could not create socket");
		return "";
	}
	if (debug == 1)
		printf("Socket created\n");

	server.sin_addr.s_addr = inet_addr("127.0.0.1");
	server.sin_family = AF_INET;
	server.sin_port = htons(9090);

	//Connect to remote server
	if (connect(sock, (struct sockaddr *) &server, sizeof(server)) < 0) {
		perror("connect failed. Error");
		return "";
	}

	strncat(message, "\n", 1);

	if (debug == 1)
		printf("Sending : %s", message);

	//Send data
	if (send(sock, message, strlen(message), 0) < 0) {
		puts("Send failed");
		return "";
	}

	memset(server_reply, '0', sizeof(server_reply));
	if ((n = read(sock, server_reply, sizeof(server_reply) - 1)) > 0) {
		server_reply[n] = 0;
	}

	close(sock);
	return server_reply;
}

/**
 * Look for State Transitions
 */
int tail(int n)
{
	FILE *in = fopen("/var/log/daemon.log", "r");
	char needle[] = "State transition";
	int count = 0, newbtstatus = -1;
    unsigned long long pos;
    char str[2*SIZE];

    if (in == NULL)
    	return -1;

    // Go to End of file
    if (fseek(in, 0, SEEK_END))
        perror("fseek() failed");
    else
    {
        pos = ftell(in);
        while (pos)
        {
            if (!fseek(in, --pos, SEEK_SET))
            {
                if (fgetc(in) == '\n')
                    if (count++ == n)
                        break;
            }
            else
                perror("fseek() failed");
        }
        printf("Reading last %d lines\n", n);
        while (fgets(str, sizeof(str), in)){
        	if (strstr(str, needle)) {
        		if (strstr(str, "1 -> 2") || strstr(str, "0 -> 1") || strstr(str, "2 -> 2")) {
        			newbtstatus = 1; // now playing
        		} else if (strstr(str, "2 -> 0")) {
        			newbtstatus = 0; // now stopped
        		}
        	}
        }
    }
    fclose(in);
    return newbtstatus;
}

/**
 * Get Information about the connection status of Bluetooth in Pulseaudio
 */
void progressChanges() {
	sleep(1);

	char output[500];

	int newbtstatus = tail(10);

	if(newbtstatus < 0 || newbtstatus == btstatus)
		// No Changes
		return;

	if (debug == 1) {
		printf("NEW Bluetooth state: %i | OLD Bluetooth state: %i\n", newbtstatus, btstatus);
	}

	// get Main PlayerID from Config
	getPlayerParameters();

	if (newbtstatus > 0) {
		printf("Running - Start Stream on Squeezebox Server\n");

		// Connect through socket and Save Status before Bluetooth is playing
		// mode ?
		sprintf(output, "%s mode ?", macaddress);
		answer = sendToSqueezeboxServer(output);
		if (strstr(answer, "play")) {
			lastmode = "play";
		} else {
			lastmode = "stop";
		}
		if (debug == 1)
			printf("Lastmode: %s\n", lastmode);

		// path ?
		sprintf(output, "%s path ?", macaddress);
		answer = sendToSqueezeboxServer(output);

		char *getpath;
		getpath = strstr(answer, "path");
		if (getpath) {
			sprintf(lastpath, "%s", getpath + 5);
		} else {
			sprintf(lastpath, "");
		}

		if (debug == 1)
			printf("Lastpath: %s\n", lastpath);

		// ADD: playlist insert -> move index +1
		// REMOVE: playlist deleteitem -> move index -1

		// Start Playing from Bluetooth by modifying Playlist
		sprintf(output, "%s playlist insert wavin:bluetooth", macaddress);
		answer = sendToSqueezeboxServer(output);

		sprintf(output, "%s playlist index +1", macaddress);
		answer = sendToSqueezeboxServer(output);

		sprintf(output, "%s playlist play", macaddress);
		answer = sendToSqueezeboxServer(output);

	} else if (newbtstatus == 0) {
		// Only switch to other state, if former state was playing
		printf("Off - Stop Stream on Squeezebox Server (Restore Stream before Bluetooth)\n");

		// Reenable Setup before Bluetooth
		//sprintf(output, "%s playlist insert %s", macaddress, lastpath);
		//answer = sendToSqueezeboxServer(output);

		sprintf(output, "%s playlist deleteitem wavin:bluetooth", macaddress);
		answer = sendToSqueezeboxServer(output);

		sprintf(output, "%s playlist index -1", macaddress);
		answer = sendToSqueezeboxServer(output);

		if (lastmode == "play") {
			sprintf(output, "%s playlist play", macaddress);
			answer = sendToSqueezeboxServer(output);
		} else {
			// Pause Player
			sprintf(output, "%s stop", macaddress);
			answer = sendToSqueezeboxServer(output);
		}

	} else {
		printf("Initialize Bluetooth Status\n");
	}
	btstatus = newbtstatus;
	fflush(stdout);
}

int main(int argc, char *argv[]) {
	printf("Usage: %s [-d]\n", argv[0]);
	printf(" -d Enable DEBUG output\n\n");

	int opt;
	while ((opt = getopt(argc, argv, "dc")) != -1) {
		switch (opt) {
		case 'd':
			debug = 1;
			break;
		case 'c':
			break; // change config Path
		default:
			break;
		}
	}

	int length, wd;
	int fd;
	char buffer[BUF_LEN];

	/* Initialize Inotify*/
	fd = inotify_init();
	if (fd < 0) {
		printf("Couldn't initialize inotify\n");
	}

	/* add watch logfile of bluealsa */
	wd = inotify_add_watch(fd, "/var/log/daemon.log", IN_MODIFY);

	if (wd == -1) {
		printf("Couldn't add watch\n");
	} else {
		printf("Watching...\n");
	}

	/* do it forever*/
	while (1) {
		length = read(fd, buffer, BUF_LEN);
		if (length < 0) {
			perror("read");
		}
		struct inotify_event *event = (struct inotify_event *) &buffer[0];
		if (event->mask & IN_MODIFY) {
			printf("Logfile was modified.\n");
			progressChanges();
		}
	}

	/* Clean up*/
	inotify_rm_watch(fd, wd);
	close(fd);

	return 0;

}

