#!/bin/bash
# Bluetooth Autostart
# Script to call from start_audioplayer.sh
# Automatically connects to Bluetooth-Speakers
# Working also with Multisqueeze 

# Separate by Debian version!
VERSION=$(lsb_release -a 2>/dev/null | grep "Codename" | sed "s/Codename:\t//")
if [ "$VERSION" = "jessie" ]; then
	# Jessie / Stretch
	USER=$(grep -a "SYSTEM_USER" /opt/max2play/audioplayer.conf | sed -n -e 's/^[A-Z_]*\=//p')
	if [ "$USER" == "pi" ]; then
		#Connect to correct pulse PID
		export XDG_RUNTIME_DIR=/run/user/1000
	fi
	running_xserver=$(ps -Al | grep "lxsession\|startx\|Xorg" | wc -l)	
	if [ "1" -gt "$running_xserver" ]; then				
		echo "start X-Server"
		export DISPLAY=':0'
		sudo su -l $USER -c startx > /dev/null 2>&1 &
		sleep 5
		# kill start_pulseaudio process
	fi	
	
	BLUETOOTHSINK=$(grep -a "BLUETOOTHSINK" /opt/max2play/options.conf | sed -n -e 's/^[A-Z_]*\=//p')
	# TODO: get current default sink and compare
	if [ ! "" = "$BLUETOOTHSINK" ]; then
		# should happen automatically by bluetooth module - but sometimes it doesn't
		sudo su -l $USER -c "pactl set-default-sink $BLUETOOTHSINK"
		echo "set Bluetoothsink to $BLUETOOTHSINK"
		
		# Special Fix for Amazon Echo to use SINK and not SOURCE a2dp Mode
		if [ "$(sudo su -l $USER -c "pacmd list-cards | grep \"\\\"Echo-\" | wc -l")" -gt "0" ]; then 
		  CARD=$(echo $BLUETOOTHSINK | sed 's/sink/card/')
		  sudo su -l $USER -c "pacmd set-card-profile $CARD a2dp"
		fi
				
		# Change sink for current playing stream
		active_index=$(sudo su -l $USER -c "pacmd list-sink-inputs | grep index | sed 's/[^0-9]*//' | head -1")
		playing_index=$(sudo su -l $USER -c "pacmd list-sink-inputs | grep $BLUETOOTHSINK | wc -l")
		if [ ! "" = "$active_index" -a "$playing_index" -lt "1" ]; then
			sudo su -l $USER -c "pacmd move-sink-input $active_index $BLUETOOTHSINK"
			echo "Change active Sink index $active_index to $BLUETOOTHSINK"
			# Crackling Pulseaudio Squeezelite on NOT Bluetooth Sink?
			# TODO Add FIX: /usr/bin/pacmd move-sink-input $(/usr/bin/pacmd list-sink-inputs | grep "index" | sed 's/[^0-9]*//') 0
			# better restart squeezelite (if stream was running when BT-connection was lost)
			sleep 1
			playing_index=$(sudo su -l $USER -c "pacmd list-sink-inputs | grep $BLUETOOTHSINK | wc -l")
			if [ "$playing_index" -gt "0" ]; then
				# Successfull changed Stream - now Restart Player
				# normal Squeezelite OR Multisqueeze instance
				MULTISQUEEZE=$(grep -e 'MULTISQUEEZE_PARAMETER_[0-9]=-o pulse' /opt/max2play/audioplayer.conf) 
				if [ ! "" = "$MULTISQUEEZE" ]; then 
					MSPLAYER=$(echo $MULTISQUEEZE | sed 's/MULTISQUEEZE_PARAMETER_//;s/=.*//')
					/etc/init.d/multisqueeze stop $MSPLAYER
					sleep 2
					/etc/init.d/multisqueeze start $MSPLAYER
				else
					/etc/init.d/squeezelite stop
					sleep 2
					/etc/init.d/squeezelite start
				fi
			else
				# TODO: no Bluetooth-Sink Available -> Stop Squeezelite from Playing Stream, if setting is enabled on Bluetooth Setup Page
				# Improvement: Pulseaudio Module?
				echo "TODO: shutdown Player?"
			fi
		fi		
	fi
	
	# Program to activate Playback on Squeezebox Server on Playback state / connected state
	# run pulsebluetoth as Daemon if it is not running currently and set to autostart
	pulsebluetoothrunning=$(ps -Al | grep pulseblue | wc -l)
	PLUSEBLUETOOTH_AUTOSTART_SBS=$(grep -a "PLUSEBLUETOOTH_AUTOSTART_SBS" /opt/max2play/autostart.conf | sed -n -e 's/^[A-Z_]*\=//p')
	if [ "$pulsebluetoothrunning" -lt "1" -a "$PLUSEBLUETOOTH_AUTOSTART_SBS" -gt "0" ]; then
		CURRENTDIR="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"
		PLUSEBLUETOOTH_DEBUG=$(grep -a "PLUSEBLUETOOTH_DEBUG" /opt/max2play/autostart.conf | sed -n -e 's/^[A-Z_]*\=//p')
		if [ "$PLUSEBLUETOOTH_DEBUG" -gt "0" ]; then
		   DEBUGFLAG="-d"
		else
		   DEBUGFLAG=""
		fi   
		sudo su -l $USER -c "$CURRENTDIR/pulsebluetooth $DEBUGFLAG > /tmp/pulsebluetooth 2>&1 &"
	fi 

elif [ "$VERSION" = "stretch" -o "$VERSION" = "buster" ]; then
    # Squeezeboxserver Auto Play / Pause on Bluetooth Streaming with Bluealsa
	alsabluetoothrunning=$(ps -Al | grep alsabluetoothsb | wc -l)
	ALSABLUETOOTH_AUTOSTART_SBS=$(grep -a "ALSABLUETOOTH_AUTOSTART_SBS" /opt/max2play/autostart.conf | sed -n -e 's/^[A-Z_]*\=//p')
	if [ "$alsabluetoothrunning" -lt "1" -a "$ALSABLUETOOTH_AUTOSTART_SBS" -gt "0" ]; then
		CURRENTDIR="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"
		ALSABLUETOOTH_DEBUG=$(grep -a "ALSABLUETOOTH_DEBUG" /opt/max2play/autostart.conf | sed -n -e 's/^[A-Z_]*\=//p')
		if [ "$ALSABLUETOOTH_DEBUG" -gt "0" ]; then
		   DEBUGFLAG="-d"
		else
		   DEBUGFLAG=""
		fi   
		sudo su -l $USER -c "$CURRENTDIR/alsabluetoothsbs $DEBUGFLAG > /tmp/alsabluetoothsbs 2>&1 &"
	fi 
	
else
	# Wheezy
	DEVICEMAC=$(bt-device -l | grep -o -e "([0-9ABCDEF:]\{17\})" | sed 's@(@@;s@)@@')
	# connected?
	CONNECTED=$(bt-device -i $DEVICEMAC | grep "Connected: 1" | wc -l)
	if [ "$CONNECTED" -lt "1" ]; then
		echo "try to connect to BT-device"
		bt-audio -c $DEVICEMAC
		sleep 2
		CONNECTED=$(bt-device -i $DEVICEMAC | grep "Connected: 1" | wc -l)
		if [ "$CONNECTED" -gt "0" ]; then
			MULTISQUEEZE=$(grep -e 'MULTISQUEEZE_PARAMETER_[0-9]=-o bluetoothspeaker' /opt/max2play/audioplayer.conf) 
			if [ ! "" = "$MULTISQUEEZE" ]; then 
				MSPLAYER=$(echo $MULTISQUEEZE | sed 's/MULTISQUEEZE_PARAMETER_//;s/=.*//')
				/etc/init.d/multisqueeze stop $MSPLAYER
				sleep 2
				/etc/init.d/multisqueeze start $MSPLAYER
			else
				/etc/init.d/squeezelite stop
				sleep 2
				/etc/init.d/squeezelite start
			fi
		fi			
	fi
fi

exit 0
