#!/bin/bash
# Installer for Bluetooth

VERSION=$(lsb_release -a 2>/dev/null | grep "Codename" | sed "s/Distributor ID:\t//;s/Codename:\t//")

# Path to current Directory
CURRENTDIR="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"

# Jessie:
# http://www.nikolaus-lueneburg.de/2015/11/bluetooth-autoplay-script/
# https://bbs.archlinux.org/viewtopic.php?id=172261
# https://github.com/ev3dev/ev3dev/issues/198

apt-get update
if [ "$VERSION" == "jessie" ]; then 
	apt-get install bluez bluez-tools -y	
	#https://github.com/ev3dev/ev3dev/issues/198
	apt-get install -o Dpkg::Options::="--force-confold" pulseaudio pulseaudio-module-bluetooth -y
	apt-get install pi-bluetooth -y
	# pactl list | grep a2dp
	# pactl list | grep bluez_sink
	# pactl set-card-profile 1 a2dp
	# pactl set-default-sink bluez_sink.40_EF_4C_FF_03_22
	
	if [ -e /etc/pulse/daemon.conf -a "$(grep -i '^default-fragments = 5' /etc/pulse/daemon.conf | wc -l)" -lt "1" ]; then
       echo -e "default-fragments = 5\ndefault-fragment-size-msec = 2\n" >> /etc/pulse/daemon.conf
    fi
elif [ "$VERSION" == "stretch" -o "$VERSION" == "buster" ]; then
	#wget http://ftp.de.debian.org/debian/pool/non-free/f/fdk-aac/libfdk-aac-dev_0.1.4-2+b1_armhf.deb 	 
    #wget http://ftp.de.debian.org/debian/pool/non-free/f/fdk-aac/libfdk-aac1_0.1.4-2+b1_armhf.deb     
    #dpkg -i libfdk-aac1_0.1.4-2+b1_armhf.deb 
    #dpkg -i libfdk-aac-dev_0.1.4-2+b1_armhf.deb
    
	# Switch to Armhf build on Zero to support all versions
	# created from Github fdk-aac compiled on RPi Zero from https://github.com/mstorsjo/fdk-aac
	pushd /opt
	wget http://cdn.max2play.com/bluealsa/fdk_2.0-1_armhf.deb
	dpkg -i fdk_2.0-1_armhf.deb
	ldconfig
	
	# TODO: for buster only bluealsa from official repository 
    apt-get install pi-bluetooth -y
    apt-get install bluetooth -y
	apt-get install bluez bluez-tools libsbc-dev libasound2-dev libbluetooth-dev libbsd-dev libglib2.0-dev libncurses5-dev -y
	apt-get install libsbc-dev -y
		
	wget http://cdn.max2play.com/bluealsa/build_1.3.1-1_armhf.deb
	dpkg -i build_1.3.1-1_armhf.deb
	
	if [ "1" == "2" ]; then 
		# use prebuild binaries instead...
		pushd /opt
		cd /opt
		git clone https://github.com/Arkq/bluez-alsa.git
	    cd bluez-alsa/
	    autoreconf --install
	    mkdir build
	    cd build
	    # check and delete existing build 
	    make clean
	    ../configure --enable-aac --enable-debug
		make && make check
		make install
	fi
	
	# Needed in startup of bluez-alsa
	chmod 644 CURRENTDIR/bluealsa.service
	chmod 644 $CURRENTDIR/bluealsa_aplay.service
	cp $CURRENTDIR/bluealsa.service /etc/systemd/system/
	cp $CURRENTDIR/bluealsa_aplay.service /etc/systemd/system/
		
	# Default Start von bluetooth deaktivieren wegen BUG: https://github.com/RPi-Distro/pi-bluetooth/issues/8
	# Startscript von Bluealsa ebenfalls angepasst!
	systemctl stop bluetooth
	systemctl disable bluetooth
	
	# bluealsa als Dienst Starten
	systemctl daemon-reload
	systemctl enable bluealsa
	systemctl start bluealsa
	
    echo "Set bluealsa to Autostart"
    # Add -C 5 to command line option Squeezelite    
    #echo ""
    #echo "Updating Squeezelite Commandline Option to disconnect audiodevice on stop: "
    #echo "Adding '-C 5' to command line options of Squeezelite"
    #SQUEEZELITE_PARAMETER_SUSPEND=$(grep -a "SQUEEZELITE_PARAMETER" /opt/max2play/audioplayer.conf | sed -n -e 's/^[A-Z_]*\=//p' | grep "\-C " | wc -l)
    #if [ "$SQUEEZELITE_PARAMETER_SUSPEND" -lt "1" ]; then
    #	sed -i "/SQUEEZELITE_PARAMETER/s/$/ -C 5/" /opt/max2play/audioplayer.conf
    #fi
    echo ""
    
    # Overwrite old Pulseaudio Version of Bluetooth-Check in Squeezelite
    cp -f /var/www/max2play/application/plugins/bluetooth/scripts/btcheck.sh /opt/squeezelite/btcheck.sh
    
    # bluetoothctl zur Verbindung nutzen
    
    # Alternative Installer for BT-Speaker (better use bluealsa player)
    # echo ""
    # echo "Install BT-Speaker Software for Streaming Audio TO Max2Play"
    # bash <(curl -s https://raw.githubusercontent.com/lukasjapan/bt-speaker/master/install.sh)
    # Add Options to Configure BT-Speaker on Bluetooth Page...
    # Edit /etc/bt_speaker/config.ini
else
	apt-get install bluez-utils -y
	apt-get install bluez-tools -y
fi

if [ -e /home/pi ]; then
	USER=pi
else
	USER=odroid
fi

# Add to Group
usermod -G bluetooth -a $USER 

# Add Socket Connection
if [ "$VERSION" == "jessie" ]; then 
	echo "You need to choose pulse as audio output for your audioplayer AND run a X-Server (e.g. Jivelite - workaround for pulseaudio)"
elif [ "$VERSION" == "stretch" -o "$VERSION" == "buster" ]; then
	if [ "$(grep 'bluetooth ' /etc/asound.conf | wc -l)" -lt "1" ]; then
		# add to /etc/asound.conf
		echo -e 'pcm.bluetooth {\n type plug\n slave.pcm {\n   type bluealsa\n   interface hci0\n   device 40:EF:4C:FF:03:22\n   profile a2dp\n   }\n hint {\n   show on\n   description "Your Bluetooth device 40:EF:4C:FF:03:22"\n   }\n}' >> /etc/asound.conf 
	fi
	
	echo "You need to choose bluetooth as audio output for your audioplayer AND make sure Bluealsa as Service running in Background (set Autostart on this page)."
else
	BLUETOOTHCONFIG=/etc/bluetooth/audio.conf
	if [ "$(grep 'Enable=Socket' $BLUETOOTHCONFIG | wc -l)" -lt "1" ]; then
		sed -i "s@\[General\]@\[General\]\nDisable=Media\nEnable=Source,Sink,Socket\n@" $BLUETOOTHCONFIG
	fi
	if [ "$(grep 'bluetoothspeaker' /etc/asound.conf | wc -l)" -lt "1" ]; then
		# add to /etc/asound.conf
		echo -e 'pcm.bluetoothspeaker {\n type plug\n slave {\n  pcm {\n   type bluetooth\n   #bluetoothdeviceid\n   device 40:EF:4C:FF:03:22\n   profile "auto"\n  }\n }\n hint {\n  show on\n  description "Max2Play bluetooth speaker"\n }\n}\nctl.bluetoothspeaker {\n   type bluetooth\n}' >> /etc/asound.conf 
	fi	
fi


echo "Finished - now connect the Bluetooth dongle or Reboot <b><a href='/plugins/max2play_settings/controller/Basic.php?action=reboot&redirecturl=/plugins/bluetooth/controller/Setup.php'>Click here to reboot now</a></b>"

exit 0