/**
 * pulsebluetooth.c 
 * Stefan Rick Max2Play.com
 * Compiled with gcc -o pulsebluetooth pulsebluetooth.c -lpulse
 */

/***
    Modified status reader to set Squeezeboxserver to Play from Bluetooth Source.

    This file is based on pacat.c and pavuctl.c,  part of PulseAudio.
    pacat.c:
    Copyright 2004-2006 Lennart Poettering
    Copyright 2006 Pierre Ossman <ossman@cendio.se> for Cendio AB

    PulseAudio is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published
    by the Free Software Foundation; either version 2.1 of the License,
    or (at your option) any later version.

    PulseAudio is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
    General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with PulseAudio; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA.
***/


#include <stdio.h>
#include <string.h>
#include <pulse/pulseaudio.h>
#include <unistd.h>
#include <sys/select.h>
#include <sys/socket.h>
#include <arpa/inet.h> 


#define _(x) x

int ret;
int btstatus = -1;
int debug = 0;
int poweroffdelay = 1;

char macparameter[] = "BLUETOOTH_MACADDRESS_MAINPLAYER";
char macaddress[30];
char *lastmode, *answer;
char lastpath[256];
char server_reply[500];

pa_context *context;

void getPlayerParameters(){
	char *token;
	char *search = "=";
	
	static const char filename[] = "/opt/max2play/audioplayer.conf";
	FILE *file = fopen ( filename, "r" );
	if ( file != NULL )
	{
	  char line [ 528 ]; 
	  while ( fgets ( line, sizeof line, file ) != NULL )
	  {
	    // Parameter Name
	    token = strtok(line, search);
	    if(strcmp(token, macparameter) == 0){
	    	// Parameter Value
	    	token = strtok(NULL, search);
	    	token[strcspn(token, "\n")] = 0;
	    	strcpy(macaddress, token);
	    }
	  }
	  fclose ( file );
	}
	return;
}

void show_error(char *s) {
    fprintf(stderr, "%s\n", s);
}

/**
	Send to Socket of Squeezebox Server
*/
char * sendToSqueezeboxServer(char *message){
    int sock, n;    
    struct sockaddr_in server;    

    //Create socket
    sock = socket(AF_INET , SOCK_STREAM , 0);
    if (sock == -1)
    {
        printf("Could not create socket");
        return "";
    }
    if(debug == 1)
    	printf("Socket created\n");

    server.sin_addr.s_addr = inet_addr("127.0.0.1");
    server.sin_family = AF_INET;
    server.sin_port = htons( 9090 );
 
    //Connect to remote server
    if (connect(sock , (struct sockaddr *)&server , sizeof(server)) < 0)
    {
        perror("connect failed. Error");
        return "";
    }  
    
    strncat(message, "\n", 1);    
	
	if(debug == 1)
    	printf("Sending : %s", message);

    //Send data
    if( send(sock , message , strlen(message) , 0) < 0)
    {
        puts("Send failed");
        return "";
    }

    memset(server_reply, '0',sizeof(server_reply));
    if ( (n = read(sock, server_reply, sizeof(server_reply)-1)) > 0)
    {      
      server_reply[n] = 0;      
    }

    close(sock);
    return server_reply;
}


/**
 * Get Information about the connection status of Bluetooth in Pulseaudio
 */
void source_cb(pa_context *c, const pa_source_info *i, int eol, void *userdata) {
    if (eol > 0) {
		return;
    }

    if(debug == 1){
    	printf("Source: name %s, description %s\n", i->name, i->description);
        printf("STATE: %i\n",  i->state);
    }
    
    char needle[] = "bluez";
    char output[500];
    if(strstr(i->name, needle)) {
	    
		// get Main PlayerID from Config
		getPlayerParameters();
		
		if(i->state == 0){
		    poweroffdelay = 1;
		    printf("Running - Start Stream on Squeezebox Server\n");		    		    		   
		    
		    // Connect through socket and Save Status before Bluetooth is playing
		    // mode ?
		    sprintf(output, "%s mode ?", macaddress);
		    answer = sendToSqueezeboxServer(output);
		    if(strstr(answer, "play")){
		    	lastmode = "play";
		    }else{
		    	lastmode = "stop";
		    }
		    if(debug == 1)
		    	printf("Lastmode: %s\n", lastmode);
		    	
		    // path ?
		    sprintf(output, "%s path ?", macaddress);
		    answer = sendToSqueezeboxServer(output);
		    
		    char *getpath;
            getpath = strstr(answer, "path");
            if(getpath){
                sprintf(lastpath, "%s", getpath + 5);
            }else{
                sprintf(lastpath, "");
            }
		    
		    if(debug == 1)
		    	printf("Lastpath: %s\n", lastpath);
		    
		    // ADD: playlist insert -> move index +1
		    // REMOVE: playlist deleteitem -> move index -1
		    
		    // Start Playing from Bluetooth by modifying Playlist		    
		    sprintf(output, "%s playlist insert wavin:%s", macaddress, i->name);
		    answer = sendToSqueezeboxServer(output);
		    
		    sprintf(output, "%s playlist index +1", macaddress);
		    answer = sendToSqueezeboxServer(output);
		    
		    sprintf(output, "%s playlist play", macaddress);
		    answer = sendToSqueezeboxServer(output);
			    
		}else if(btstatus == 0){
	        // Only switch to other state, if former state was playing
	        
	        
	        // FIX: Make sure to trigger only when Streaming really stopped and not only because of a short interruption
	        // RECheck status of Bluetooth after short delay...
	        if(poweroffdelay == 1){
	        	sleep(2);
	        	if(debug == 1){
		    		printf("Waiting for Power Off\n");
		        	printf("STATE: %i\n",  i->state);
		    	}
	        	pa_operation *o;
			    if (!(o = pa_context_get_source_info_by_index(c, i->index, source_cb, NULL))) {
					show_error(("pa_context_get_source_info_by_index() failed"));
					return;
			    }
			    pa_operation_unref(o);
			    poweroffdelay = 2;
			    return;
			}
	        
	        
	        printf("Off - Stop Stream on Squeezebox Server (Restore Stream before Bluetooth)\n");			
			
			// Reenable Setup before Bluetooth
			//sprintf(output, "%s playlist insert %s", macaddress, lastpath);
		    //answer = sendToSqueezeboxServer(output);
		    
		    sprintf(output, "%s playlist deleteitem wavin:%s", macaddress, i->name);
		    answer = sendToSqueezeboxServer(output);
		    
		    sprintf(output, "%s playlist index -1", macaddress);
		    answer = sendToSqueezeboxServer(output);
		    
			if (lastmode == "play") {    
			    sprintf(output, "%s playlist play", macaddress);
			    answer = sendToSqueezeboxServer(output);
			}else{
				// Pause Player
				sprintf(output, "%s stop", macaddress);				
				answer = sendToSqueezeboxServer(output);
			}

		}else{
			printf("Initialize Bluetooth Status\n");
		}
        btstatus = i->state;
        fflush(stdout);
    }   
}

void subscribe_cb(pa_context *c, pa_subscription_event_type_t t, uint32_t index, void *userdata) {

    switch (t & PA_SUBSCRIPTION_EVENT_FACILITY_MASK) {
	
	    case PA_SUBSCRIPTION_EVENT_SOURCE:
			if ((t & PA_SUBSCRIPTION_EVENT_TYPE_MASK) == PA_SUBSCRIPTION_EVENT_REMOVE)
			    printf("Removing source index %d\n", index);
			else {
			    pa_operation *o;
			    if (!(o = pa_context_get_source_info_by_index(c, index, source_cb, NULL))) {
				show_error(("pa_context_get_source_info_by_index() failed"));
				return;
			    }
			    pa_operation_unref(o);
			}
			break;
    }
}

void context_state_cb(pa_context *c, void *userdata) {

    switch (pa_context_get_state(c)) {
	    case PA_CONTEXT_UNCONNECTED:
	    case PA_CONTEXT_CONNECTING:
	    case PA_CONTEXT_AUTHORIZING:
	    case PA_CONTEXT_SETTING_NAME:
	break;

    case PA_CONTEXT_READY: {
	pa_operation *o;


	if (!(o = pa_context_get_source_info_list(c,
						  source_cb,
						  NULL
						  ))) {
	    show_error(("pa_context_subscribe() failed"));
	    return;
	}
	pa_operation_unref(o);

	pa_context_set_subscribe_callback(c, subscribe_cb, NULL);

	if (!(o = pa_context_subscribe(c, (pa_subscription_mask_t)
				       (PA_SUBSCRIPTION_MASK_SOURCE), NULL, NULL))) {
	    show_error(("pa_context_subscribe() failed"));
	    return;
	}
	pa_operation_unref(o);

	break;
    }

    case PA_CONTEXT_FAILED:
    case PA_CONTEXT_TERMINATED:
    default:
	return;
    }
}

int main(int argc, char *argv[]) {
	printf("Usage: %s [-d]\n", argv[0]);
	printf(" -d Enable DEBUG output\n\n");
	
	int opt;	
	while ((opt = getopt(argc, argv, "dc")) != -1) {
        switch (opt) {
	        case 'd': debug = 1; break;
	        case 'c': break; // change config Path       
	        default:
	            break;
	        }
    }
	
    // Define pulse audio loop and connection variables
    pa_mainloop *pa_ml;
    pa_mainloop_api *pa_mlapi;
    pa_operation *pa_op;
    pa_time_event *time_event;

    // Create a mainloop API and connection to the default server
    pa_ml = pa_mainloop_new();
    pa_mlapi = pa_mainloop_get_api(pa_ml);
    context = pa_context_new(pa_mlapi, "Device list");

    // This function connects to the pulse server
    pa_context_connect(context, NULL, 0, NULL);

    // This function defines a callback so the server will tell us its state.
    pa_context_set_state_callback(context, context_state_cb, NULL);	
    
    fflush(stdout);
    if (pa_mainloop_run(pa_ml, &ret) < 0) {
	    printf("pa_mainloop_run() failed.");
	    exit(1);
    }
}


