#!/bin/bash

# Separate by Debian version - Not available for Wheezy

# Path to current Directory
CURRENTDIR="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"

echo ""

VERSION=$(lsb_release -a 2>/dev/null | grep "Codename" | sed "s/Codename:\t//")
if [ "$VERSION" == "jessie" -o  "$VERSION" == "stretch" -o  "$VERSION" == "buster" ]; then
	# Jessie
	
	#Check for telnet
	TELNETINSTALL=$(dpkg -s telnet | grep installed)
	if [ "$TELNETINSTALL" == "" ]; then 
		echo "Install Telnet"
		apt-get update
		apt-get install telnet zip -y
	fi
	
	USER=$(grep -a "SYSTEM_USER" /opt/max2play/audioplayer.conf | sed -n -e 's/^[A-Z_]*\=//p')
	if [ "$USER" == "pi" ]; then
		#Connect to correct pulse PID
		export XDG_RUNTIME_DIR=/run/user/1000
	fi
	running_xserver=$(ps -Al | grep "lxsession\|startx\|Xorg" | wc -l)	
	if [ "1" -gt "$running_xserver" ]; then				
		echo "start X-Server"
		export DISPLAY=':0'
		sudo su -l $USER -c startx > /dev/null 2>&1 &
		sleep 10		
	fi		
	
	sudo -E -H -u $USER pacmd load-module module-null-sink
	sudo -E -H -u $USER pacmd set-default-sink null
	
	echo ""
	echo "Install Waveinput Plugin"
	wget http://cdn.max2play.com/WaveInputLinux-v104_7.6.ZIP -O /opt/max2play/cache/waveinput.zip
	
	if [ "$VERSION" == "jessie" ]; then
		unzip -o /opt/max2play/cache/waveinput.zip -d /opt/max2play/cache/
		echo "Add some magic..."
		cp -f $CURRENTDIR/custom-convert.conf /opt/max2play/cache/WaveInput/	
		if [ ! "$USER" == "pi" ]; then
			sed -i 's/pi/$USER/' /opt/max2play/cache/WaveInput/custom-convert.conf
		fi
		# Repack
		pushd /opt/max2play/cache/
		zip -r waveinput.zip WaveInput
	fi
	
	# Edit Waveinput to accept LMS 8.x - Already done...
	
	cp /opt/max2play/cache/waveinput.zip /var/lib/squeezeboxserver/cache/DownloadedPlugins/WaveInput.zip
	chown squeezeboxserver /var/lib/squeezeboxserver/cache/DownloadedPlugins/WaveInput.zip	
	echo "WaveInput: needs-install" >> /var/lib/squeezeboxserver/prefs/plugin/state.prefs
	
	foundWaveInput=$(grep -i "WaveInput" /var/lib/squeezeboxserver/prefs/plugin/extensions.prefs | wc -l)
	if [ "$foundWaveInput" -lt "1" ]; then
		sed -i 's/plugin:$/plugin:\n  WaveInput: 1/' /var/lib/squeezeboxserver/prefs/plugin/extensions.prefs
		sed -i 's/plugin: {}/plugin:\n  WaveInput: 1/' /var/lib/squeezeboxserver/prefs/plugin/extensions.prefs
	fi
	echo "WaveInput installed"	
	
	if [ "$VERSION" == "jessie" ]; then	
		BLUETOOTHSINK=$(grep -a "BLUETOOTHSINK" /opt/max2play/options.conf | sed -n -e 's/^[A-Z_]*\=//p' | sed 's/bluez_sink.//')
		if [ "$BLUETOOTHSINK" == "" ]; then
			echo ""
			echo "<b>No Bluetooth-Device is connected! Make sure to connect this first!</b>"
		else
			echo ""		
			#SBSURL=$(LANG=C && /sbin/ip addr | grep -o 'inet [0-9\.]\+' | grep -v '127.0.0.1' | grep -o '[0-9\.]\+')
			#echo "http://$SBSURL:9000/plugins/Favorites/index.html?index=0&action=editset&entrytitle=Bluetooth&entryurl=wavin:bluez_source.$BLUETOOTHSINK&editset=Speichern"
			BLUETOOTHADDRESS=$(echo "$BLUETOOTHSINK" | sed 's/_/:/g')
			BTDEVICENAME=$(/usr/bin/bt-device -l | grep "$BLUETOOTHADDRESS" | grep -o "^[^(]\+" | sed "s/ /%20/g")
			echo "Add Favorite Bluetooth for ID $BLUETOOTHSINK on Squeezebox Server (Title: Bluetooth%20$BTDEVICENAME, URL: wavin:bluez_source.$BLUETOOTHSINK)"
			echo "favorites add url:wavin:bluez_source.$BLUETOOTHSINK title:Bluetooth%20$BTDEVICENAME item_id:0" | telnet localhost 9090	
		fi	
	else
		echo "Add Favorite Bluetooth on Squeezebox Server (Title: Bluetooth URL: wavin:bluetooth)"
		echo "favorites add url:wavin:bluetooth title:Bluetooth item_id:0" | telnet localhost 9090
	fi
	echo ""
	echo "<b>Now: </b> Choose Bluetooth from favorites in Squeezebox Server and click play OR enable Autostart Bluetooth-Streaming in Squeezebox Server on this page. After this, you can stream audio from your smartphone to your Squeezebox Server and multiple players."
	
	/etc/init.d/logitechmediaserver restart
	
	#pacat --passthrough --volume=65536 --format=s16ne --channels=2 --rate=44100 -r -d bluez_source.30_75_12_9D_6C_C2 | /usr/share/squeezeboxserver/Bin/armhf-linux/flac --endian=little --sign=signed --channels=2 --bps=16 --sample-rate=44100 -cs --compression-level-0 -
else	
	echo "Wheezy Image is not supported!"
fi

##############
#pacmd load-module module-loopback
#pacmd list-sources
#pacmd list-sink-inputs
#move bluez sink input to null output
#pacmd move-sink-input 13 1

exit 0
